#oasis controller main
import sys
from PyQt5 import QtCore, QtGui, QtWidgets
from SerialGRBL import GRBL
from SerialHP45 import HP45
from Interface import Interface
import threading
import time

class MainWindow(QtWidgets.QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()        
        self.ui = Interface()
        self.ui.initUI()
        self.ui.show()
        
        self.grbl = GRBL()
        self.inkjet = HP45()
        
        #grbl connect button
        self.grbl_connection_state = 0 #connected state of grbl
        self.ui.motion_connect.clicked.connect(self.GrblConnect)
        self.ui.motion_set_port.returnPressed.connect(self.GrblConnect)
        
        #grbl send command button
        self.ui.motion_send_line.clicked.connect(self.GrblSendCommand)
        self.ui.motion_write_line.returnPressed.connect(self.GrblSendCommand)
        
        #grbl home button
        self.ui.motion_home.clicked.connect(self.grbl.Home)
        
        #grbl jog buttons
        self.ui.motion_xp.clicked.connect(lambda: self.grbl.Jog('X', '10', '6000'))
        self.ui.motion_xn.clicked.connect(lambda: self.grbl.Jog('X', '-10', '6000'))
        self.ui.motion_yp.clicked.connect(lambda: self.grbl.Jog('Y', '10', '6000'))
        self.ui.motion_yn.clicked.connect(lambda: self.grbl.Jog('Y', '-10', '6000'))
        
        #inkjet connect button
        self.inkjet_connection_state = 0 #connected state of inkjet
        self.ui.inkjet_connect.clicked.connect(self.InkjetConnect)
        self.ui.inkjet_set_port.returnPressed.connect(self.InkjetConnect)
        
        #inkjet send command button
        self.ui.inkjet_send_line.clicked.connect(self.InkjetSendCommand)
        self.ui.inkjet_write_line.returnPressed.connect(self.InkjetSendCommand)
        
        #inkjet function buttons
    
    def GrblConnect(self):
        """Gets the GRBL serial port and attempt to connect to it"""
        
        if (self.grbl_connection_state == 0): #get connection state, if 0 (not connected)
            #print("Attempting connection with GRBL")
            temp_port = str(self.ui.motion_set_port.text()) #get text
            temp_succes = self.grbl.Connect(temp_port) #attempt to connect
            if (temp_succes == 1): #on success, 
                self.ui.motion_connect.setText("Disconnect")#rewrite button text
                self.grbl_connection_state = 1 #set  state
                self.ui.motion_set_port.clear()
                #start a thread that will update the serial in and output for GRBL
                self._grbl_stop_event = threading.Event()
                self.grbl_update_thread = threading.Thread(target=self.GrblUpdate)
                self.grbl_update_thread.start()
                
            else:
                print("Connection with GRBL failed")
        else: #on state 1
            #print("disconnecting from GRBL")
            self.grbl.Disconnect() #disconnect
            self.grbl_connection_state = 0 #set state to disconnected
            self.ui.motion_connect.setText("Connect") #rewrite button
            self._grbl_stop_event.set() #close the grbl serial thread
            
    def GrblUpdate(self):
        """updates serial in and output for the GRBL window"""
        time.sleep(1)
        while not self._grbl_stop_event.is_set():
            grbl_serial_in = str(self.grbl.GetWindowInput())
            grbl_serial_out = str(self.grbl.GetWindowOutput())
            
            self.ui.motion_serial_output.moveCursor(QtGui.QTextCursor.End)
            self.ui.motion_serial_output.insertPlainText(grbl_serial_out)
            self.ui.motion_serial_output.moveCursor(QtGui.QTextCursor.End)
            
            self.ui.motion_serial_input.moveCursor(QtGui.QTextCursor.End)
            self.ui.motion_serial_input.insertPlainText(grbl_serial_in)
            self.ui.motion_serial_input.moveCursor(QtGui.QTextCursor.End)
            
            #update state and coordinates
            self.ui.motion_state.setText(self.grbl.motion_state)
            self.ui.motion_x_pos.setText(str(self.grbl.motion_x_pos))
            self.ui.motion_y_pos.setText(str(self.grbl.motion_y_pos))
            self.ui.motion_b_pos.setText(str(self.grbl.motion_z_pos))
            
            time.sleep(0.2)
            
    def GrblSendCommand(self):
        """Gets the command from the textedit and prints it to Grbl"""
        if (self.grbl_connection_state == 1):
            temp_command = str(self.ui.motion_write_line.text())#get line
            temp_command += "\r" #add end of line
            self.grbl.SerialWriteBufferRaw(temp_command) #write to grbl
            self.ui.motion_write_line.clear() #clear line

    def InkjetConnect(self):
        """Gets the inkjet serial port and attempt to connect to it"""
        if (self.inkjet_connection_state == 0): #get connection state, if 0 (not connected)
            #print("Attempting connection with HP45")
            temp_port = str(self.ui.inkjet_set_port.text()) #get text
            temp_succes = self.inkjet.Connect(temp_port) #attempt to connect
            if (temp_succes == 1): #on success, 
                self.ui.inkjet_connect.setText("Disconnect")#rewrite button text
                self.inkjet_connection_state = 1 #set  state
                self.ui.inkjet_set_port.clear()
                #start a thread that will update the serial in and output for HP45
                self._inkjet_stop_event = threading.Event()
                self.inkjet_update_thread = threading.Thread(target=self.InkjetUpdate)
                self.inkjet_update_thread.start()
                
            else:
                print("Connection with HP failed")
        else: #on state 1
            #print("disconnecting from HP45")
            self.inkjet.Disconnect() #disconnect
            self.inkjet_connection_state = 0 #set state to disconnected
            self.ui.inkjet_connect.setText("Connect") #rewrite button
            self._inkjet_stop_event.set() #close the HP45 serial thread
            
    def InkjetUpdate(self):
        """updates serial in and output for the inkjet window"""
        time.sleep(1)
        while not self._inkjet_stop_event.is_set():
            inkjet_serial_in = str(self.inkjet.GetWindowInput())
            inkjet_serial_out = str(self.inkjet.GetWindowOutput())
            
            self.ui.inkjet_serial_output.moveCursor(QtGui.QTextCursor.End)
            self.ui.inkjet_serial_output.insertPlainText(inkjet_serial_out)
            self.ui.inkjet_serial_output.moveCursor(QtGui.QTextCursor.End)
            
            self.ui.inkjet_serial_input.moveCursor(QtGui.QTextCursor.End)
            self.ui.inkjet_serial_input.insertPlainText(inkjet_serial_in)
            self.ui.inkjet_serial_input.moveCursor(QtGui.QTextCursor.End)
            
            #update state and coordinates
            self.ui.inkjet_temperature.setText(str(self.inkjet.inkjet_temperature))
            self.ui.inkjet_pos.setText(str(self.inkjet.inkjet_x_pos))
            
            
            time.sleep(0.2)
    
    def InkjetSendCommand(self):
        """Gets the command from the textedit and prints it to Inkjet"""
        if (self.inkjet_connection_state == 1):
            temp_command = str(self.ui.inkjet_write_line.text())#get line
            temp_command += "\r" #add end of line
            self.inkjet.SerialWriteBufferRaw(temp_command) #write to inkjet
            self.ui.inkjet_write_line.clear() #clear line


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    gui = MainWindow()
    sys.exit(app.exec_())


